package com.biganto.visual.roompark.conductor.mosby.mvi


import android.util.Log
import android.view.View
import androidx.annotation.NonNull
import com.bluelinelabs.conductor.Controller
import com.hannesdorfmann.mosby3.MviConductorDelegateCallback
import com.hannesdorfmann.mosby3.mvi.MviPresenter
import com.hannesdorfmann.mosby3.mvp.MvpView

/**
 *   В связи с тем, что в исходной реализации MviConductorLifeCycleListener презентер приватное поле,
 *   есть необходимость рконструировать дефолтный класс библиотеки с внесением изменений - проверкой поля presenter на
 *   null значение при вызове методов attachView и destroy.
 *
 *   Суть проблемы кроется в том, что при реализации некоторых кейсов (например, когда мы желаем сохранять стек
 *   навигации среди нескольких роутеров одновроеменно), мы сохраняем ссылку на контроллер, даже в тех случаях когда
 *   презентер уничтожен (вьюха в mosbyMVI у нас и так отвязана от жизненного цикла контроллера) и при каскадном
 *   удалении бэкстека, у нас может получиться таким образом, что презентер уже был уничтожен ранее. Т.о. необходима
 *   проверка на наличие инстанса привязанного к контроллеру презентера.
 *
 *  * **Детально по ссылке:** [mosby issue](https://github.com/sockeqwe/mosby-conductor/issues/38)
 */
class BigantoMviConductorLifecycleListener<V : MvpView, P : MviPresenter<V, *>>
/**
 * Instantiate a new Mosby MVP Listener
 *
 * @param callback [MviConductorDelegateCallback] to set presenter. Typically the
 * controller
 * himself.
 */
@JvmOverloads constructor(private var callback: MviConductorDelegateCallback<V, P>?,
                          private val keepPresenterInstance: Boolean = true) : Controller.LifecycleListener() {

    private var presenter: P? = null

    override fun postCreateView(@NonNull controller: Controller, @NonNull view: View) {

        var viewStateWillBeRestored = false

        if (presenter == null) {
            // Process death,
            // hence no presenter with the given viewState id stored, although we have a viewState id
            presenter = callback!!.createPresenter()
            if (DEBUG) {
                Log.d(
                    DEBUG_TAG,
                        "New Presenter instance created for $controller. Presenter: $presenter")
            }
        } else {
            viewStateWillBeRestored = true
            if (DEBUG) {
                Log.d(
                    DEBUG_TAG, "Reusing Presenter instance for controller "
                        + controller
                        + ". Presenter: "
                        + presenter)
            }
        }

        val viewMpv = callback?.mvpView ?: throw NullPointerException(
                "MvpView returned from getMvpView() is null. Returned by " + controller.activity!!)

        if (viewStateWillBeRestored) {
            callback!!.setRestoringViewState(true)
        }

        presenter?.attachView(viewMpv)

        if (viewStateWillBeRestored) {
            callback!!.setRestoringViewState(false)
        }

        if (DEBUG) {
            Log.d(
                DEBUG_TAG,
                    "MvpView attached to Presenter. MvpView: $view   Presenter: $presenter")
        }
    }

    override fun preDestroyView(@NonNull controller: Controller, @NonNull view: View) {
        presenter?.detachView()

        if (DEBUG) {
            Log.d(
                DEBUG_TAG, "detached MvpView from Presenter. MvpView "
                    + callback!!.mvpView
                    + "   Presenter: "
                    + presenter)
        }
    }

    override fun postDestroy(@NonNull controller: Controller) {
        if (DEBUG) {
            Log.d(
                DEBUG_TAG, "Presenter destroyed because View destroyed. MvpView "
                    + callback!!.mvpView
                    + "   Presenter: "
                    + presenter)
        }
        presenter?.destroy()
        presenter = null
        callback = null
    }

    companion object {

        private var DEBUG = false
        private const val DEBUG_TAG = "MviLifecycleListener"

        /**
         * Determines whether or not a Presenter Instance should be kept
         *
         * @param keepPresenterInstance true, if the delegate has enabled keep
         */
        internal fun retainPresenterInstance(keepPresenterInstance: Boolean, controller: Controller): Boolean {

            return keepPresenterInstance && (controller.activity!!.isChangingConfigurations
                    || !controller.activity!!.isFinishing) && !controller.isBeingDestroyed
        }
    }
}