package com.biganto.visual.roompark.data.repository.api.retrofit

import com.biganto.visual.roompark.data.repository.api.IRoomParkApi
import com.biganto.visual.roompark.data.repository.api.retrofit.raw.*
import com.biganto.visual.roompark.util.extensions.asInt
import io.reactivex.Completable
import io.reactivex.Observable
import io.reactivex.ObservableSource
import io.reactivex.ObservableTransformer
import io.reactivex.schedulers.Schedulers
import retrofit2.Response
import retrofit2.Retrofit
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 29.10.2019.
 */

const val DEFAULT_ARTICLE_PAGE_SIZE = 10

class RetrofitRepository @Inject constructor(retrofit: Retrofit) : IRoomParkApi {

    private val api = retrofit.create(IRoomParkMobileApi::class.java)

    init {
        Timber.d("Module inited")
    }

    override fun authenticate(login: String, pwd: String): Observable<AuthRaw> =
        api.authoriz(email = login, pwd = pwd)
            .subscribeOn(Schedulers.io())
            .compose(RetrofitResponseValidation())

    override fun subscribeTopic(
        userToken: String,
        deviceToken: String,
        topicName: String
    ): Observable<StatusResponse> =
        api.subscribe(token = userToken, deviceToken = deviceToken, topic = topicName)
            .compose(RetrofitResponseValidation())

    override fun unSuubscribeTopic(
        userToken: String,
        deviceToken: String,
        topicName: String
    ): Observable<StatusResponse> =
        api.unsubscribe(token = userToken, deviceToken = deviceToken, topic = topicName)
            .compose(RetrofitResponseValidation())


    override fun getDeals(userToken: String): Observable<List<DealRaw>> =
        api.getDeals(token = userToken)
            .compose(RetrofitResponseValidation())

    override fun getFavorites(userToken: String): Observable<List<EstateRaw>> =
        api.getFavorites(token = userToken)
            .compose(RetrofitResponseValidation())

    override fun getFeeds(): Observable<List<FeedRaw>> =
        api.getFeeds()
            .subscribeOn(Schedulers.io())
            .compose(RetrofitResponseValidation())

    override fun getArticlesPage(
        feedName: String,
        pageSize: Int,
        page: Int
    ): Observable<ArticlesListPaginationRaw> =
        api.getArticlesPage(alias = feedName, pageSize = pageSize, page = page)
            .compose(RetrofitResponseValidation())

    override fun getArticle(id: Int): Observable<ArticleRaw> =
        api.getArticle(id = id)
            .subscribeOn(Schedulers.io())
            .compose(RetrofitResponseValidation())

    override fun getAlbums(parentId: Int?): Observable<List<ImageAlbumRaw>> =
        api.getAlbums(id = parentId)
            .subscribeOn(Schedulers.io())
            .compose(RetrofitResponseValidation())

    override fun getWebCamsList(): Observable<List<WebCamRaw>> =
        api.getWebCams()
            .subscribeOn(Schedulers.io())
            .compose(RetrofitResponseValidation())

    override fun getPhotos(parentId: Int): Observable<List<GalleryImageRaw>> =
        api.getPhotos(id = parentId)
            .subscribeOn(Schedulers.io())
            .compose(RetrofitResponseValidation())

    override fun getEstatePlanTypes(estateId: Int): Observable<List<PlanTypeRaw>> =
        api.getPlanTypes(estateId = estateId)
            .compose(RetrofitResponseValidation())

    override fun getDirectPlan(
        estateId: Int,
        planType: Int,
        showFurniture: Boolean,
        showSizes: Boolean,
        showWalls: Boolean,
        showElectric: Boolean
    ): Observable<String> =
        api.getPlan(
            estate_id = estateId,
            planType = planType,
            furniture = showFurniture.asInt,
            sizes = showSizes.asInt,
            walls = showWalls.asInt,
            electric = showElectric.asInt
        )
            .compose(RetrofitResponseValidation())

//    fun getPlanString(
//        estateId: Int,
//        planType: Int,
//        showFurniture: Boolean,
//        showSizes: Boolean,
//        showWalls: Boolean,
//        showElectric: Boolean) = api.getPlan(
//        estate_id = estateId,
//        planType = planType,
//        furniture = showFurniture.asInt,
//        sizes = showSizes.asInt,
//        walls = showWalls.asInt,
//        electric = showElectric.asInt
//    ).req


    @Deprecated("hidden by states.getEstate method")
    override fun getMultiTour(building: Int, flat: Int): Observable<Int> =
        api.getMultiTourId(
            building = building,
            flatNumber = flat
        )
            .compose(RetrofitResponseValidation())

    override fun getEstate(building: Int, flat: Int): Observable<EstateRaw> =
        api.getEstate(
            building = building,
            flatNumber = flat
        )
            .compose(RetrofitResponseValidation())

}

enum class PlanTypeCatalog(val planId:Int, val planName:String ){
    PROJECT(0,"проектная планировка"),
    PROJECT_PLAN_1(1,"перепланировка №1"),
    PROJECT_PLAN_2(2,"перепланировка №2"),
}


internal class RetrofitResponseValidation<T> : ObservableTransformer<Response<T>, T> {
    override fun apply(responseObservable: Observable<Response<T>>): ObservableSource<T> {
        return responseObservable.switchMap { resp ->
                        Timber.d("reutrned code: %s",resp.code())
            if (resp.code() == 200 || resp.code() == 206)
                if (resp.body() == null)
                    return@switchMap Completable.complete().toObservable<T>().doOnNext{ Timber.d("Completed responseBody")}
                else return@switchMap Observable.just(resp.body())
            Timber.d("Returning Exception!")
            return@switchMap Observable.error<T>(retrofit2.HttpException(resp))
        }
    }
}