package com.biganto.visual.roompark.data.repository.db.requrey

import android.app.Application
import com.biganto.visual.roompark.data.repository.db.IDb
import com.biganto.visual.roompark.data.repository.db.requrey.model.*
import com.biganto.visual.roompark.di.dagger.DATABASE_VERSION
import dagger.Module
import dagger.Provides
import io.reactivex.Completable
import io.reactivex.Observable
import io.reactivex.Single
import io.requery.Persistable
import io.requery.android.sqlite.DatabaseSource
import io.requery.reactivex.KotlinReactiveEntityStore
import io.requery.reactivex.ReactiveResult
import io.requery.sql.KotlinEntityDataStore
import io.requery.sql.TableCreationMode
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 13.06.2018.
 */


@Module
class DbModule{

        @Provides
        fun provideStore(context: Application): KotlinReactiveEntityStore<Persistable> {
                Timber.d("Kotlin store creating..")
                val source = DatabaseSource(context, Models.DEFAULT, "BigantoPerfect", DATABASE_VERSION)

//                source.setLoggingEnabled(true)
//                source.setWriteAheadLoggingEnabled(true)
                source.setTableCreationMode(TableCreationMode.DROP_CREATE)
                val store = KotlinEntityDataStore<Persistable>(source.configuration)
                Timber.d("Kotlin store %s",source)
                return KotlinReactiveEntityStore(store)
        }
}

class RequeryRepository @Inject constructor(
        private val store: KotlinReactiveEntityStore<Persistable>
)
: IDb {

        private inline fun <reified T : Persistable> fetchAll() =
                store.select(T::class)

        override fun upsertUser(entity: UserEntity): Observable<UserEntity> =
                store.upsert(entity).toObservable()

        override fun <T : Persistable> upsert(entity: T) = store.upsert(entity)

        override fun <T : List<Persistable>> upsert(entity: T): Single<Iterable<Persistable>> =
                store.upsert(entity)

        override fun <T : List<Persistable>> blockingUpsert(entity: T) =
                store.upsert(entity).toObservable().blockingSubscribe()

        override fun upsertEstate(entity:EstateEntity) =
                store.upsert(entity).toObservable().blockingSubscribe()

        override fun fetchFeeds(): Observable<FeedEntity> =
                fetchAll<FeedEntity>().get().observable()

        override fun fetchEstateByNumber(building:Int,number:String): ReactiveResult<EstateEntity> =
                store.select(EstateEntity::class)
                        .where(EstateEntity.NUMBER.eq(number))
                        .and(EstateEntity.INFO_BUILDING.eq(building))
                        .get()

        override fun getFeed(feedAlias: String): ReactiveResult<FeedEntity> =
                fetchAll<FeedEntity>().where(FeedEntity.ALIAS.eq(feedAlias))
                        .get()

        override fun getArticle(id: Int): ReactiveResult<ArticleEntity> =
                fetchAll<ArticleEntity>().where(ArticleEntity.ID.eq(id))
                        .get()

        override fun getTopLevelAlbums(): Observable<List<ImageAlbumEntity>> =
                Observable.fromArray(
                        store.raw(
                                ImageAlbumEntity::class,
                                "SELECT alb.* from ImageAlbum alb " +
                                        "left join ImageAlbumJunction albJun on albJun.albumId = alb.id " +
                                        "WHERE albJun.albumId is null "
                        ).toList()
                )

        override fun getChildAlbums(parentId: Int): Observable<List<ImageAlbumEntity>> =
                Observable.fromArray(
                        store.raw(
                                ImageAlbumEntity::class,
                                "SELECT alb.* " +
                                "from ImageAlbum alb " +
                                "    left join ImageAlbumJunction albJun on albJun.albumId = alb.id " +
                                "WHERE albJun.parentId is $parentId"
                        ).toList()
                )

        override fun getAlbum(albumId: Int): Observable<ImageAlbumEntity> =
                store.select(ImageAlbumEntity::class)
                        .where(ImageAlbumEntity.ID.eq(albumId))
                        .get().observable()

        override fun getEstate(estateId: Int): Observable<EstateEntity> =
                store.select(EstateEntity::class)
                        .where(EstateEntity.ID.eq(estateId))
                        .get().observable()

        override fun getPhotos(albumId: Int): Observable<GalleryPhotoEntity> =
                store.select(GalleryPhotoEntity::class)
                        .where(GalleryPhotoEntity.ALBUM_ID.eq(albumId))
                        .get().observable()

        override fun getPhoto(photoId: Int): Observable<GalleryPhotoEntity> =
                store.select(GalleryPhotoEntity::class)
                        .where(GalleryPhotoEntity.ID.eq(photoId))
                        .get().observable()

        override fun fetchArticles(feedAlias: String, pageSize: Int, startIndex: Int)
                : Observable<ArticleEntity> =
                fetchAll<ArticleEntity>()
                        .where(ArticleEntity.FEED_ID.eq(feedAlias))
                        .orderBy(ArticleEntity.PUBLISHED.desc())
                        .limit(pageSize)
                        .offset(startIndex)
                        .get().observable()

        override fun checkIfExistsAlbumJunction(albumId: Int, parentAlbumId: Int) =
                fetchAll<ImageAlbumJunctionEntity>()
                        .where(ImageAlbumJunctionEntity.ALBUM_ID.eq(albumId))
                        .and(ImageAlbumJunctionEntity.PARENT_ID.eq(parentAlbumId))
                        .get()
                        .firstOrNull()

        override fun fetchUser(uuid: Int): Observable<UserEntity> =
                store.select(UserEntity::class).where(UserEntity.UUID.eq(uuid)).get().observable()
//                fetchAll<UserEntity>().where(UserEntity.UUID.eq(uuid)).get().observable()


        override fun fetchAllUsers(): Observable<List<UserEntity>> =
                store.select(UserEntity::class).get().observable().toList().toObservable()

        fun upsertFeeds(entity: List<FeedEntity>) =
                store.upsert(entity)

        override fun userObservableResult(uuid: Int) =
                store.select(UserEntity::class)
                        .where(UserEntity.UUID.eq(uuid))
                        .get()
                        .observableResult()

        override fun getUserFavorites(uuid: Int): Observable<EstateEntity> =
                store.select(EstateEntity::class)
                        .where(EstateEntity.USER_ID.eq(uuid))
                        .and(EstateEntity.FAVORITE.eq(true))
                        .get().observable()

        override fun setArticleReadState(id:Int,state:Boolean): Completable =
                store.update(ArticleEntity::class)
                        .set(ArticleEntity.READ,state)
                        .where(ArticleEntity.ID.eq(id))
                        .get()
                        .single()
                        .ignoreElement()


        override fun setDealReadState(id:String,state:Boolean): Completable =
                store.update(DealEntity::class)
                        .set(DealEntity.READ,state)
                        .where(DealEntity.ID.eq(id))
                        .get()
                        .single()
                        .ignoreElement()
}
