package com.biganto.visual.roompark.presentation.screen.feed_list

import android.view.View
import android.widget.ProgressBar
import androidx.core.os.bundleOf
import androidx.core.view.isNotEmpty
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.HeaderToolbarModel
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.presentation.screen.article.ArticleScreenController
import com.biganto.visual.roompark.presentation.screen.feed_list.util.ArticlesAdapter
import com.biganto.visual.roompark.util.extensions.setGone
import com.biganto.visual.roompark.util.view_utils.grid.CeilsDecoration
import com.bluelinelabs.conductor.RouterTransaction
import com.google.android.material.textview.MaterialTextView
import com.jakewharton.rxbinding3.recyclerview.scrollStateChanges
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import timber.log.Timber
import java.util.concurrent.TimeUnit
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

const val FEED_ALIAS = "DIRECT_FEED_ID_KEY"

class ArticlesScreenController  :
    BigantoBaseController<ArticlesScreenViewState
        , ArticlesScreen
        , ArticlesScreenPresenter>
    , ArticlesScreen {


    constructor()

    constructor(feedAlias:String):super(bundleOf(FEED_ALIAS to feedAlias))

    override fun requsetsNewArticles(): Observable<Int> =
        articlesRecyclerView.scrollStateChanges()
            .filter { it == RecyclerView.SCROLL_STATE_IDLE}
            .map { articlesRecyclerView.layoutManager as LinearLayoutManager }
            .map { it.findLastCompletelyVisibleItemPosition() }
            .filter{ articlesRecyclerView.isNotEmpty() }
            .filter { it>(articlesRecyclerView.adapter?.itemCount?:0)-4 }
            .map { articlesRecyclerView.adapter?.itemCount?:0 }
            .debounce(120L, TimeUnit.MILLISECONDS)
            .observeOn(AndroidSchedulers.mainThread())

    override fun injectDependencies() {
        getComponent()
    }

    @Inject
    override lateinit var injectedPresenter: ArticlesScreenPresenter

    @BindView(R.id.articles_recycler_view)
    lateinit var articlesRecyclerView: RecyclerView

    @BindView(R.id.progress_bar_view)
    lateinit var progressBar: ProgressBar

    @BindView(R.id.empty_list_text_view)
    lateinit var emptyListNotice: MaterialTextView


    private fun setToolbar() {
        articlesRecyclerView.isNestedScrollingEnabled = false

        toolBar.setToolbar(
            HeaderToolbarModel(
                true, null
                , resources?.getString(
                    when (args.getString(FEED_ALIAS)) {
                        "news" -> R.string.news_header
                        else -> R.string.blog_header
                    }
                ), true
            )
            , null
        )
    }

    private fun bindRecycler() {
        articlesRecyclerView.isNestedScrollingEnabled = true
        articlesRecyclerView.layoutManager =
            LinearLayoutManager(activity, RecyclerView.VERTICAL, false)
        articlesRecyclerView.adapter = ArticlesAdapter()
        articlesRecyclerView.itemAnimator = null

        if (articlesRecyclerView.itemDecorationCount == 0)
            articlesRecyclerView.addItemDecoration(
                CeilsDecoration(
                    1
                    , resources?.getDimensionPixelSize(R.dimen.ceil_grid_padding)
                )
            )

        detachDisposable.add(
            (articlesRecyclerView.adapter as ArticlesAdapter)
                .onItemClicked.subscribe {
                router.pushController(RouterTransaction.with(ArticleScreenController(it.articleId)))
            }
        )
    }

    override fun onViewBound(v: View) {
        bottomNavigationController.hide()
        setToolbar()
        bindRecycler()
    }

    override fun render(viewState: ArticlesScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is ArticlesScreenViewState.Idle -> render(viewState)
            is ArticlesScreenViewState.ArticlesLoaded -> render(viewState)
            is ArticlesScreenViewState.ArticlesScrollPage -> render(viewState)
            is ArticlesScreenViewState.RestoreView -> render(viewState)
            is ArticlesScreenViewState.SomeError -> render(viewState)
        }
    }

    private fun render(viewState: ArticlesScreenViewState.Idle){
        progressBar.setGone(false)
        emptyListNotice.setGone(true)
        articlesRecyclerView.setGone(true)
    }

    private fun render(viewState: ArticlesScreenViewState.SomeError) {
        showError(viewState.exception)
        progressBar.setGone(true)
    }


    private fun render(viewState: ArticlesScreenViewState.ArticlesScrollPage){
        Timber.d("got items: ${viewState.items.size}")
        (articlesRecyclerView.adapter as ArticlesAdapter).addItems(viewState.items)
    }

    private fun render(viewState: ArticlesScreenViewState.RestoreView){
        progressBar.setGone(true)
        emptyListNotice.setGone(true)
        articlesRecyclerView.setGone(true)
        if (viewState.restore.articles.isNullOrEmpty())
        {
            emptyListNotice.setGone(false)
            articlesRecyclerView.setGone(true)
        }
        else{
            emptyListNotice.setGone(true)
            articlesRecyclerView.setGone(false)
            (articlesRecyclerView.adapter as ArticlesAdapter).setItems(viewState.restore.articles)
        }
    }



    private fun render(viewState: ArticlesScreenViewState.ArticlesLoaded) {
        progressBar.setGone(true)
        emptyListNotice.setGone(true)
        articlesRecyclerView.setGone(true)
        if (viewState.items.isNullOrEmpty())
        {
            emptyListNotice.setGone(false)
            articlesRecyclerView.setGone(true)
        }
        else{
            emptyListNotice.setGone(true)
            articlesRecyclerView.setGone(false)
            (articlesRecyclerView.adapter as ArticlesAdapter).setItems(viewState.items)
        }

    }

    private fun getComponent() = DaggerArticlesScreenComponent.factory()
        .create(args.getString(FEED_ALIAS)!!
            ,RoomParkApplication.component,activity as RoomParkMainActivity)
        .inject(this)

    override fun getLayoutId(): Int = R.layout.feed_direct_screen

    override fun handleBack(): Boolean {
        Timber.d("handle back in class ")
        router.popController(this)
        return true
    }


}