package com.biganto.visual.roompark.presentation.screen.home.home_routing

import android.os.Bundle
import android.util.SparseArray
import android.view.Menu
import android.view.MenuItem
import android.view.View
import androidx.annotation.IdRes
import androidx.annotation.NonNull
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.presentation.screen.home.HomeScreen
import com.biganto.visual.roompark.presentation.screen.home.HomeScreenPresenter
import com.biganto.visual.roompark.presentation.screen.home.HomeScreenViewState
import com.bluelinelabs.conductor.ChangeHandlerFrameLayout
import com.bluelinelabs.conductor.Controller
import com.bluelinelabs.conductor.Router
import com.bluelinelabs.conductor.RouterTransaction
import com.bluelinelabs.conductor.changehandler.FadeChangeHandler
import com.google.android.material.bottomnavigation.BottomNavigationView
import timber.log.Timber


/**
 * The [Controller] for the Bottom Navigation View. Populates a [BottomNavigationView]
 * with the supplied [Menu] resource. The first item set as checked will be shown by default.
 * The backstack of each [MenuItem] is switched out, in order to maintain a separate backstack
 * for each [MenuItem] - even though that is against the Google Design Guidelines.
 *
 *
 *
 * Internally works similarly to [com.bluelinelabs.conductor.support.RouterPagerAdapter],
 * in the sense that it keeps track of the currently active [MenuItem] and the paired Child
 * [Router]. Everytime we navigate from one to another, or [ ][Controller.onSaveInstanceState] is called, we save the entire instance state of the Child
 * [Router], and cache it, so we have it available when we navigate to another [ ] and can then restore the correct Child [Router] (and thus the entire backstack)
 *
 * @author chris6647@gmail.com
 * @see [Material
 * Design Guidelines](https://material.io/guidelines/components/bottom-navigation.html.bottom-navigation-behavior)
 */



abstract class BottomNavigationController :
    BigantoBaseController<HomeScreenViewState
            , HomeScreen
            , HomeScreenPresenter>
    , HomeScreen {



    lateinit var controllerContainer: ChangeHandlerFrameLayout

    protected var currentlySelectedItemId = INVALID_INT

    protected var routerSavedStateBundles: SparseArray<Bundle>? = null

    /**
     * Used to popToTag (this) [BottomNavigationController] instance if needed
     *
     * @return The tag set on the transaction contains this [BottomNavigationController]
     * instance
     */
    protected abstract val tag: String

    constructor():super()

    constructor(args: Bundle) : super(args)

    override fun onViewBound(v: View) {
        retainViewMode=RetainViewMode.RELEASE_DETACH
    }

    @IdRes
    protected abstract fun getStartupMenuId(): Int

    protected override fun onAttach(@NonNull view: View) {
        super.onAttach(view)
        Timber.d(" OnAttachView: ${this::class}")
        if (routerSavedStateBundles == null) {
            /*
       * Fresh start, setup everything.
       * Must be done in onAttach to avoid artifacts if using multiple Activities,
       * and in case of resuming the app (i.e. when the view is not created again)
       */
            val menu = bottomNavigationController.bottomNavigation.menu
            val menuSize = menu.size()
            routerSavedStateBundles = SparseArray(menuSize)
            val startUpMenuItem = menu.findItem(getStartupMenuId())
            for (i in 0 until menuSize) {

                val menuItem = menu.getItem(i)
                /* We want start from direct menu item */
                if (menuItem==startUpMenuItem) {
                    /*
           * Seems like the BottomNavigationView always initializes index 0 as isChecked / Selected,
           * regardless of what was set in the menu xml originally.
           * So basically all we're doing here is always setting up menuItem index 0.
           */
                    val itemId = menuItem.itemId
                    bottomNavigationController.bottomNavigation.selectedItemId = itemId
                    navigateTo(getControllerFor(itemId), false)
                    break
                }
            }
        } else {
            /*
       * Since we are restoring our state,
       * and onRestoreInstanceState is called before onViewBound,
       * all we need to do is rebind.
       */
            getChildRouter(currentlySelectedItemId)!!.rebindIfNeeded()
        }
    }

    /**
     * Get the Child [Router] matching the supplied ItemId.
     *
     * @param menuItemId [MenuItem] ID
     * @return
     */
    protected fun getChildRouter(@IdRes menuItemId: Int): Router? {
        return getChildRouter(controllerContainer, "itemId:$menuItemId")
    }

    /**
     * Remove the supplied [Router] as a child of this Controller.
     *
     * @param childRouter
     */
    protected fun destroyChildRouter(childRouter: Router?) {
        if (childRouter!=null) removeChildRouter(childRouter)
    }

    /**
     * Navigate to the supplied [Controller], while setting the menuItemId as selected on the
     * [BottomNavigationView]. Supplied Controller must match the nextMenuItemId as defined in
     * [BottomNavigationMenuItem] or an [IllegalArgumentException] will be thrown.
     *
     *
     *
     * If this method is to be called when Requires [BottomNavigationController] is not
     * attached, it requires [BottomNavigationController.getTag] to return a valid tag, that
     * will return this [BottomNavigationController] instance in this [Router]'s
     * backstack.
     *
     * @param controller              to navigate to
     * @param shouldOverrideBackstack whether or not to navigate to whatever existing backstack, or to
     * set a new one
     */
    fun navigateTo(@NonNull controller: Controller, shouldOverrideBackstack: Boolean) {
        if (isSupportedController(controller)) {
            if (isAttached) {
                @IdRes val menuItemId = getIdForController(controller)
                if (currentlySelectedItemId != menuItemId) {
                    /* Navigating to new menuItemId */
                    if (currentlySelectedItemId != INVALID_INT) {
                        /* Save state of current router before destroying it */
                        val oldChildRouter = getChildRouter(currentlySelectedItemId)
                        save(oldChildRouter!!, currentlySelectedItemId)
                        destroyChildRouter(oldChildRouter)
                    }
                    /* Configure the new childRouter */
                    val newChildRouter = getChildRouter(menuItemId)
                    val routerSavedState = routerSavedStateBundles!!.get(menuItemId)
                    if (!shouldOverrideBackstack
                            && routerSavedState != null
                            && !routerSavedState.isEmpty) {
                        newChildRouter!!.restoreInstanceState(routerSavedState)
                        routerSavedStateBundles!!.remove(menuItemId)
                        newChildRouter.rebindIfNeeded()
                    } else {
                        newChildRouter!!.setRoot(RouterTransaction.with(controller))
                    }

                    ensureMenuSelectionState(menuItemId)
                } else if (currentlySelectedItemId != INVALID_INT) {
                    /* Navigating to same menuItemId */
                    @IdRes val newMenuItem = if (shouldOverrideBackstack) menuItemId else currentlySelectedItemId
                    /* Don't want to save state, because we are resetting it */
                    destroyChildRouter(getChildRouter(newMenuItem))
                    routerSavedStateBundles!!.remove(newMenuItem)
                    /* Must get reference to newly recreated childRouter to avoid old view not being removed */
                    getChildRouter(newMenuItem)!!
                            .setRoot(
                                    RouterTransaction.with(controller)
//                                            .popChangeHandler(HorizontalChangeHandler(false))
//                                            .pushChangeHandler(FadeChangeHandler(false))
                            )
                    if (newMenuItem != currentlySelectedItemId) {
                        ensureMenuSelectionState(newMenuItem)
                    }
                } else {
                    Timber.e("Attempted to reset backstack on BottomNavigationController with currentlySelectedItemId=$currentlySelectedItemId")
                }
            } else {
                /*
         * Navigate to ourselves, and once the view is ready (so we can get the childRouter),
         * navigate as instructed.
         */
                this@BottomNavigationController.addLifecycleListener(
                        object : Controller.LifecycleListener() {
                            override fun postAttach(@NonNull attachedController: Controller, @NonNull view: View) {
                                super.postAttach(attachedController, view)
                                this@BottomNavigationController.removeLifecycleListener(this)
                                this@BottomNavigationController.navigateTo(controller, shouldOverrideBackstack)
                            }

                            override fun preDestroy(@NonNull controller: Controller) {
                                super.preDestroy(controller)
                                /* Clean ourselves up in case we're destroyed without having been attached */
                                this@BottomNavigationController.removeLifecycleListener(this)
                            }
                        })
                router.popToTag(tag)
            }
        } else {
            Timber.e("Attempted to navigate to unsupported controller=$controller")
        }
    }

    /**
     * Resets the backstack for the menuItemId matching the [Controller] found at backstack
     * index 0.
     *
     * @param backstack
     */
    fun navigateTo(@NonNull backstack: List<RouterTransaction>) {
        val rootController = backstack[0].controller()
        if (isSupportedController(rootController)) {
            val menuItemId = getIdForController(rootController)
            if (isAttached()) {
                destroyChildRouter(getChildRouter(currentlySelectedItemId))
                routerSavedStateBundles!!.remove(currentlySelectedItemId)
                if (currentlySelectedItemId != menuItemId) {
                    ensureMenuSelectionState(menuItemId)
                }
                /* Must get reference to newly recreated childRouter to avoid old view not being removed */
                getChildRouter(currentlySelectedItemId)!!.setBackstack(backstack, FadeChangeHandler())
            } else {
                /*
         * Navigate to ourselves, and once the view is ready (so we can get the childRouter),
         * navigate as instructed.
         */
                this@BottomNavigationController.addLifecycleListener(
                        object : Controller.LifecycleListener() {
                            override fun postAttach(@NonNull controller: Controller, @NonNull view: View) {
                                super.postAttach(controller, view)
                                this@BottomNavigationController.removeLifecycleListener(this)
                                this@BottomNavigationController.navigateTo(backstack)
                            }

                            override fun preDestroy(@NonNull controller: Controller) {
                                super.preDestroy(controller)
                                /* Clean ourselves up in case we're destroyed without having been attached */
                                this@BottomNavigationController.removeLifecycleListener(this)
                            }
                        })
                getRouter().popToTag(tag)
            }
        } else {
            Timber.e("Attempted to navigate to backstack with unsupported root controller=$rootController")
        }
    }

    /**
     * Ensure correct Checked state based on given menuItemId
     *
     * @param menuItemId
     */
    private fun ensureMenuSelectionState(@IdRes menuItemId: Int) {
        val menu = bottomNavigationController.bottomNavigation.menu
        for (i in 0 until menu.size()) {
            val menuItem = menu.getItem(i)
            if (menuItem.isChecked && menuItem.itemId != menuItemId) {
                menuItem.isChecked = false
            } else if (menuItem.itemId == menuItemId) {
                menuItem.isChecked = true
                currentlySelectedItemId = menuItemId
            }
        }
        if (currentlySelectedItemId != menuItemId) {
            Timber.e("Unexpected BottomNavigation selected Menu Item.")
        }
    }

    /**
     * Saves the Child [Router] into a [Bundle] and caches that [Bundle].
     *
     *
     *
     * Be cautious as this call causes the controller flag it needs reattach, so it should only be
     * called just prior to destroying the router
     *
     * @param menuItemId [MenuItem] ID
     */
    private fun save(childRouter: Router, @IdRes menuItemId: Int) {
        val routerBundle = Bundle()
        childRouter.saveInstanceState(routerBundle)
        routerSavedStateBundles!!.put(menuItemId, routerBundle)
    }

    override fun onRestoreInstanceState(@NonNull savedInstanceState: Bundle) {
        Timber.d(" onRestoreInstanceState: ${this::class}")
        routerSavedStateBundles = savedInstanceState.getSparseParcelableArray(
            KEY_STATE_ROUTER_BUNDLES
        )
        currentlySelectedItemId = savedInstanceState.getInt(KEY_STATE_CURRENTLY_SELECTED_ID, INVALID_INT)
    }

    override fun onSaveInstanceState(@NonNull outState: Bundle) {
        Timber.d(" onSaveInstanceState: ${this::class}")
        outState.putSparseParcelableArray(KEY_STATE_ROUTER_BUNDLES, routerSavedStateBundles)
        /*
     * For some reason the BottomNavigationView does not seem to correctly restore its
     * selectedId, even though the view appears with the correct state.
     * So we keep track of it manually
     */
        outState.putInt(KEY_STATE_CURRENTLY_SELECTED_ID, currentlySelectedItemId)
    }


    /**
     * Return a target instance of [Controller] for given menu item ID
     *
     * @param menuItemId the ID tapped by the user
     * @return the [Controller] instance to navigate to
     */
    @NonNull
    protected abstract fun getControllerFor(@IdRes menuItemId: Int): Controller

    /**
     * @param controller
     * @return the MenuItemId for matching the given [Controller]
     */
    @IdRes
    protected abstract fun getIdForController(@NonNull controller: Controller): Int

    /**
     * @param controller [Controller] to test
     * @return whether or not the supplied controller is of a class supported in this [ ]
     */
    protected abstract fun isSupportedController(@NonNull controller: Controller): Boolean

    companion object {

        val TAG = "BottomNavigationContr"

        private val KEY_MENU_RESOURCE = "key_menu_resource"
        private val KEY_STATE_ROUTER_BUNDLES = "key_state_router_bundles"
        private val KEY_STATE_CURRENTLY_SELECTED_ID = "key_state_currently_selected_id"
        val INVALID_INT = -1
    }
}